from typing import List
from costack_temporal_sdk.context.function_context import FunctionContext
import pprint

class RuntimeContext:
    def __init__(self, debug):
        self._debug = debug
        self._entry_context = dict()
        self._exit_context = dict()
        self._steps: List[FunctionContext] = []
    
    def __repr__(self):
        return pprint.pformat({
            "debug": self.debug,
            "entry_context": self.entry_context,
            "exit_context": self.exit_context,
            "steps": self.steps
        })

    @property
    def debug(self):
        return self._debug
    @property
    def steps(self):
        return self._steps
    @property
    def entry_context(self):
        return self._entry_context
    @property
    def exit_context(self):
        return self._exit_context
    
    def add_step(self, step):
        self._steps.append(step)
    
    def add_entry_context(self, key, context):
        self._entry_context[key] = context
    def add_exit_context(self, key, context):
        self._exit_context[key] = context
    



import uuid
import pprint

class FunctionContext:
    TYPE = "function_context"

    def __init__(self, start_time, end_time, arguments, keyword_arguments, function_return=None, exec_traceback=None):
        self._start_time = start_time
        self._end_time = end_time
        self._arguments = arguments
        self._keyword_arguments = keyword_arguments
        self._function_return = function_return
        self._exec_traceback = exec_traceback
        self._step_id = uuid.uuid4().hex
    
    def __repr__(self):
        return pprint.pformat({
            "start_time": self._start_time,
            "end_time": self._end_time,
            "arguments": self._arguments,
            "keyword_arguments": self._keyword_arguments,
            "function_return": self._function_return,
            "exec_traceback": self._exec_traceback,
            "step_id": self._step_id
        })



import json

class CostackRequest:
    def __init__(self, event):
        # initialie 
        self._header = None
        self._http_method = None
        self._body = None 
        self._is_base64_encoded = None 
        self._route_path = None 
        
        # build from the event input 
        self.build_from_event(event)

    @property
    def header(self):
        return self._header

    @property
    def http_method(self):
        return self._http_method

    @property
    def body(self):
        return self._body

    @property
    def is_base64_encoded(self):
        return self._is_base64_encoded

    @property
    def route_path(self):
        return self._route_path

    def build_from_event(self, event):
        # parse the input event 
        event = event['event']
        http = event['requestContext']['http']
        self._http_method = http['method']
        self._header = event['headers']
        self._is_base64_encoded = event['isBase64Encoded']
        self._route_path = event["routeKey"].split()[1]
        if self._http_method.lower() == "get":
            # do nothing 
            pass 
        elif self._http_method.lower() == "post":
            self._body = event['body']
    
def costack_http(methods=[]):
    for m in methods:
        if m.lower() not in ["get", "post", "put"]:
            raise ValueError(f"unexpected http method {m}")
    def decorator(function):
        def wrapper(event, context):
            #parse the input event to a readable format 
            # check the method and make sure it matches 
            request = CostackRequest(event)
            if request.http_method not in methods:
                raise ValueError(f"unmatched http method {m} for function")
            result = function(request, context)
            return result
        return wrapper
    return decorator

@costack_http(methods=["GET", "POST"])
def simple_lambda(event, context):
    print(event.header)
    print(event.body)
    return event 

if __name__=="__main__":
    with open("sample_get.json", "r") as f:
        event = json.loads(f.read())
    res = simple_lambda(event,22)
    print(res)




import config
import os 
from costack_temporal_sdk.constants import DEBUG_ENVIRON_KEY, LAMBDA_EVENT_KEY, LAMBDA_CONTEXT_KEY, LAMBDA_RETURN_KEY
from costack_temporal_sdk.context.runtime_context import RuntimeContext

def lambda_runtime(handler):
    config.runtime = RuntimeContext(os.environ.get(DEBUG_ENVIRON_KEY) == "True")
    def wrapper_handler(event, context):
        config.runtime.add_entry_context(LAMBDA_EVENT_KEY, event)
        config.runtime.add_entry_context(LAMBDA_CONTEXT_KEY, context)
        lambda_return = handler(event, context)
        config.runtime.add_exit_context(LAMBDA_RETURN_KEY, lambda_return)
        return lambda_return
    return wrapper_handler



import traceback
import time
import config
from costack_temporal_sdk.context.function_context import FunctionContext
from costack_temporal_sdk.context.runtime_context import RuntimeContext

def step(function, *args, **kwargs):
    # global RUNTIME_CONTEXT
    if config.runtime.debug:
        return function(*args, **kwargs)
    
    start_time = time.time()
    try:
        function_return = function(*args, **kwargs)
        end_time = time.time()
        function_context = FunctionContext(start_time, end_time, list(args), dict(kwargs), function_return = function_return)
        config.runtime.add_step(function_context)
    except Exception as err:
        end_time = time.time()
        exec_traceback = traceback.format_exc()
        function_context = FunctionContext(start_time, end_time, list(args), dict(kwargs), exec_traceback = exec_traceback)
        config.runtime.add_step(function_context)
        raise err
    return function_return
